import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { forwardRef, useEffect, useImperativeHandle, useState } from "react";
import { useTranslation } from "react-i18next";
import styled from "styled-components";
import Spacing from "../../../../../Styled/Spacing";
import { TextSpan } from "../../../../../Styled/Text";
import { Category, ShareAction } from "../../../../../Core/AnalyticEvents/analyticEvents";
import Clipboard from "../../../../Clipboard";
import { buildShareLink, buildShortShareLink } from "../BuildShareLink";
import { ShareUrlWarning } from "./ShareUrlWarning";
export const ShareUrl = forwardRef(function ShareUrl({ terria, viewState, includeStories, shouldShorten, children, theme, inputTheme, rounded, callback }, forwardRef) {
    const { t } = useTranslation();
    const [shareUrl, setShareUrl] = useState("");
    const [shorteningInProgress, setShorteningInProgress] = useState(false);
    const [placeholder, setPlaceholder] = useState();
    useImperativeHandle(forwardRef, () => ({
        url: shareUrl,
        shorteningInProgress: shorteningInProgress
    }), 
    /* eslint-disable-next-line react-hooks/exhaustive-deps */
    [forwardRef, shareUrl, shorteningInProgress]);
    useEffect(() => {
        if (shouldShorten) {
            setPlaceholder(t("share.shortLinkShortening"));
            setShorteningInProgress(true);
            buildShortShareLink(terria, viewState, {
                includeStories
            })
                .then((shareUrl) => setShareUrl(shareUrl))
                .catch(() => {
                setShareUrl(buildShareLink(terria, viewState, {
                    includeStories
                }));
            })
                .finally(() => setShorteningInProgress(false));
        }
        else {
            setShareUrl(buildShareLink(terria, viewState, {
                includeStories
            }));
        }
        /* eslint-disable-next-line react-hooks/exhaustive-deps */
    }, [terria, viewState, shouldShorten, includeStories]);
    return (_jsxs(_Fragment, { children: [_jsx(Explanation, { textDark: theme === "light", children: t("clipboard.shareExplanation") }), _jsx(Spacing, { bottom: 1 }), _jsx(Clipboard, { theme: theme, text: shareUrl, inputPlaceholder: placeholder, inputTheme: inputTheme, rounded: rounded, onCopy: (text) => {
                    var _a;
                    return (_a = terria.analytics) === null || _a === void 0 ? void 0 : _a.logEvent(Category.share, ShareAction.storyCopy, text);
                } }), children, _jsx(Spacing, { bottom: 2 }), _jsx(ShareUrlWarning, { terria: terria, viewState: viewState, callback: callback || (() => { }) })] }));
});
const Explanation = styled(TextSpan) `
  opacity: 0.8;
`;
//# sourceMappingURL=ShareUrl.js.map